//operation of the dac8501

// SYNC : p2.5
// SCLK : P2.4
// DIN  : p2.3



#include"msp430g2253.h"


//high or low level of the ctrl pin

#define normal 0x00                    //normal working mode
#define out_1K 0x01                         //output 1K ou
#define out_100K 0x02                       //output 100K ou
#define HIGH_Z 0x03                         //HIGHimpedence output 

#define FSYNC_L P2OUT &= ~BIT5                 
#define FSYNC_H P2OUT |= BIT5
#define SCLK_L  P2OUT &= ~BIT4
#define SCLK_H  P2OUT |= BIT4
#define DIN_L P2OUT &= ~BIT3
#define DIN_H P2OUT |= BIT3

float vol = 0.5;                             //voltage of the dac output

unsigned int vol_code;                      //voltage code of the output

void cal_vol_code(float);                   //calculate the voltage code
void set_8501(unsigned char,unsigned int);  //set the data to the dac8501 shift register

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;                 //stop WDT
  P2DIR |= BIT3 + BIT4 +BIT5;               //set p2.3 2.4 2.5 to output direction
  cal_vol_code(vol);                        
  set_8501(normal,vol_code);
}



void set_8501(unsigned char mode,unsigned int code)  //send the working mode and the voltage code
{
   FSYNC_L;                                //pull down the SYNC
   SCLK_H;                                 //pull up the SCLK and ready to clock the data on the falling edge of the sclk
   unsigned char TEMP = 0x80;              //send the first byte which include the working mode of the chip
   unsigned int TEMP1 = 0x8000;            //send the next two byte which contains the voltage code
   unsigned char i;                        //loop value
   for(i=0;i<8;i++)
   {
     SCLK_H;
     if((mode & TEMP) == TEMP) DIN_H;      //corresponding bit equals one
     else DIN_L;
      TEMP >>= 1;
     SCLK_L;
   }
   
   for(i=0;i<16;i++)
   {
     SCLK_H;
     if((code & TEMP1) == TEMP1) DIN_H;    //corresponding bit equals one
     else DIN_L;
      TEMP1 >>= 1;
     SCLK_L;
   }
   
   SCLK_H;
   FSYNC_H;                                //pull up again
 
}


void cal_vol_code(float voltage)
{
  vol_code = voltage * 65536 / 4.096;          //voltage = Vref / 65536 * code
}